@@ -23,8 +23,9 @@ from drafting.pens.outlinepen import OutlinePen
 from drafting.pens.translationpen import TranslationPen, polarCoord
 
 from drafting.beziers import CurveCutter, splitCubicAtT
-
 from drafting.interpolation import norm
+from drafting.grid import Grid
+
 
 class DraftingPen(RecordingPen, SHContext):
     """Fluent subclass of RecordingPen"""
@@ -151,6 +152,17 @@ class DraftingPen(RecordingPen, SHContext):
             return True
         return False
     
+    def fully_close_path(self):
+        if self.value[-1][0] == "closePath":        
+            start = self.value[0][-1][-1]
+            end = self.value[-2][-1][-1]
+
+            if start != end:
+                self.value = self.value[:-1]
+                self.lineTo(start)
+                self.closePath()
+        return self
+    
     def reverse(self):
         """Reverse the winding direction of the pen."""
         if self.unended():
@@ -294,7 +306,13 @@ class DraftingPen(RecordingPen, SHContext):
         return self
 
     def addComponent(self, glyphName, transformation):
-        super().addComponent(glyphName, transformation)
+        if hasattr(self, "_glyphSet") and self._glyphSet:
+            if glyphName in self._glyphSet:
+                dp = DraftingPen().glyph(self._glyphSet[glyphName], self._glyphSet).transform(transformation)
+                self.record(dp)
+            #super().addComponent(glyphName, transformation)
+        else:
+            print("no glyphset, cannot add component")
         return self
 
     def replay(self, pen):
@@ -310,25 +328,32 @@ class DraftingPen(RecordingPen, SHContext):
             pen.replay(self)
         return self
     
-    def glyph(self, glyph):
+    def glyph(self, glyph, glyphSet=None):
         """Play a glyph (like from `defcon`) into this pen."""
+        if glyphSet:
+            self._glyphSet = glyphSet
         glyph.draw(self)
         return self
     
-    def to_glyph(self, name=None, width=None):
+    def to_glyph(self, name=None, width=None, allow_blank=False):
         """
         Create a glyph (like from `defcon`) using this pen’s value.
         *Warning*: if path is unended, closedPath will be called
         """
         from defcon import Glyph
-        if self.unended():
-            self.closePath()
+        if not allow_blank:
+            if self.unended():
+                self.closePath()
         bounds = self.bounds()
         glyph = Glyph()
         glyph.name = name
         glyph.width = width or bounds.w
-        sp = glyph.getPen()
-        self.replay(sp)
+        try:
+            sp = glyph.getPen()
+            self.replay(sp)
+        except AssertionError:
+            if not allow_blank:
+                print(">>>blank glyph:", glyph.name)
         return glyph
     
     def rect(self, rect):
@@ -958,6 +983,11 @@ class DraftingPen(RecordingPen, SHContext):
         for k, v in kwargs.items():
             self.macros[k] = v
         return self
+
+    def guide(self, grid:Grid):
+        for k, v in grid.keyed.items():
+            setattr(self, k, v)
+        return self
     
     def print(self, *args):
         for a in args:
@@ -992,7 +1022,9 @@ class DraftingPen(RecordingPen, SHContext):
         """
         return fn(self, *args)
     
-    def cond(self, condition, if_true: Callable[["DraftingPen"], None], if_false=Callable[["DraftingPen"], None]):
+    def cond(self, condition, if_true:Callable[["DraftingPen"], None], if_false:Callable[["DraftingPen"], None]=None):
+        if callable(condition):
+            condition = condition(self)
         # TODO make if_false optional
         if condition:
             if callable(if_true):
@@ -1000,7 +1032,7 @@ class DraftingPen(RecordingPen, SHContext):
             else:
                 self.gs(if_true)
         else:
-            if if_false:
+            if if_false is not None:
                 if callable(if_false):
                     if_false(self)
                 else:
@@ -1075,9 +1107,13 @@ class DraftingPen(RecordingPen, SHContext):
                 elif k == "stroke":
                     existing = attrs.get("stroke", {})
                     if not isinstance(v, dict):
-                        attrs[k] = dict(color=normalize_color(v), weight=existing.get("weight", 1))
+                        c = normalize_color(v)
+                        default_w = 1 if c.a > 0 else 0
+                        attrs[k] = dict(color=c, weight=existing.get("weight", default_w))
                     else:
-                        attrs[k] = dict(weight=v.get("weight", existing.get("weight", 1)), color=normalize_color(v.get("color", 0)))
+                        c = normalize_color(v.get("color", 0))
+                        default_w = 1 if c.a > 0 else 0
+                        attrs[k] = dict(weight=v.get("weight", existing.get("weight",default_w)), color=c)
                 elif k == "strokeWidth":
                     if "stroke" in attrs:
                         attrs["stroke"]["weight"] = v
